import sys
import typing
import bmesh.types
import mathutils
import bpy.types


def average_vert_facedata(bm: 'bmesh.types.BMesh',
                          verts: typing.List['bmesh.types.BMVert']):
    ''' Average Vertices Facevert Data. Merge uv/vcols associated with the input vertices at the bounding box center. (I know, it's not averaging but the vert_snap_to_bb_center is just too long).

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    '''

    pass


def beautify_fill(
        bm: 'bmesh.types.BMesh', faces: typing.List['bmesh.types.BMFace'],
        edges: typing.List['bmesh.types.BMEdge'], use_restrict_tag: bool,
        method: typing.Union[int, str]) -> dict:
    ''' Beautify Fill. Rotate edges to create more evenly spaced triangles.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param edges: edges that can be flipped
    :type edges: typing.List['bmesh.types.BMEdge']
    :param use_restrict_tag: restrict edge rotation to mixed tagged vertices
    :type use_restrict_tag: bool
    :param method: method to define what is beautiful
    :type method: typing.Union[int, str]
    :rtype: dict
    :return: - geom : new flipped faces and edges **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def bevel(bm: 'bmesh.types.BMesh', geom: typing.
          Union[typing.List['bmesh.types.BMEdge'], typing.
                List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']],
          offset: float, offset_type: typing.Union[int, str],
          profile_type: typing.Union[int, str], segments: int, profile: float,
          affect: typing.Union[int, str], clamp_overlap: bool, material: int,
          loop_slide: bool, mark_seam: bool, mark_sharp: bool,
          harden_normals: bool, face_strength_mode: typing.Union[int, str],
          miter_outer: typing.Union[int, str],
          miter_inner: typing.Union[int, str], spread: float,
          smoothresh: float, custom_profile: 'bpy.types.bpy_struct',
          vmesh_method: typing.Union[int, str]) -> dict:
    ''' Bevel. Bevels edges and vertices

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param geom: input edges and vertices
    :type geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param offset: amount to offset beveled edge
    :type offset: float
    :param offset_type: how to measure the offset
    :type offset_type: typing.Union[int, str]
    :param profile_type: The profile type to use for bevel.
    :type profile_type: typing.Union[int, str]
    :param segments: number of segments in bevel
    :type segments: int
    :param profile: profile shape, 0->1 (.5=>round)
    :type profile: float
    :param affect: Whether to bevel vertices or edges.
    :type affect: typing.Union[int, str]
    :param clamp_overlap: do not allow beveled edges/vertices to overlap each other
    :type clamp_overlap: bool
    :param material: material for bevel faces, -1 means get from adjacent faces
    :type material: int
    :param loop_slide: prefer to slide along edges to having even widths
    :type loop_slide: bool
    :param mark_seam: extend edge data to allow seams to run across bevels
    :type mark_seam: bool
    :param mark_sharp: extend edge data to allow sharp edges to run across bevels
    :type mark_sharp: bool
    :param harden_normals: harden normals
    :type harden_normals: bool
    :param face_strength_mode: whether to set face strength, and which faces to set if so
    :type face_strength_mode: typing.Union[int, str]
    :param miter_outer: outer miter kind
    :type miter_outer: typing.Union[int, str]
    :param miter_inner: outer miter kind
    :type miter_inner: typing.Union[int, str]
    :param spread: amount to offset beveled edge
    :type spread: float
    :param smoothresh: for passing mesh's smoothresh, used in hardening
    :type smoothresh: float
    :param custom_profile: CurveProfile
    :type custom_profile: 'bpy.types.bpy_struct'
    :param vmesh_method: The method to use to create meshes at intersections.
    :type vmesh_method: typing.Union[int, str]
    :rtype: dict
    :return: - faces : output faces **type** list of ( bmesh.types.BMFace ) - edges : output edges **type** list of ( bmesh.types.BMEdge ) - verts : output verts **type** list of ( bmesh.types.BMVert )
    '''

    pass


def bisect_edges(bm: 'bmesh.types.BMesh',
                 edges: typing.List['bmesh.types.BMEdge'], cuts: int,
                 edge_percents: dict) -> dict:
    ''' Edge Bisect. Splits input edges (but doesn't do anything else). This creates a 2-valence vert.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param cuts: number of cuts
    :type cuts: int
    :param edge_percents: Undocumented.
    :type edge_percents: dict
    :rtype: dict
    :return: - geom_split : newly created vertices and edges **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def bisect_plane(
        bm: 'bmesh.types.BMesh', geom: typing.
        Union[typing.List['bmesh.types.BMEdge'], typing.
              List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']],
        dist: float, plane_co: typing.List['mathutils.Vector'],
        plane_no: typing.List['mathutils.Vector'], use_snap_center: bool,
        clear_outer: bool, clear_inner: bool) -> dict:
    ''' Bisect Plane. Bisects the mesh by a plane (cut the mesh in half).

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param geom: input geometry
    :type geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param dist: minimum distance when testing if a vert is exactly on the plane
    :type dist: float
    :param plane_co: point on the plane
    :type plane_co: typing.List['mathutils.Vector']
    :param plane_no: direction of the plane
    :type plane_no: typing.List['mathutils.Vector']
    :param use_snap_center: snap axis aligned verts to the center
    :type use_snap_center: bool
    :param clear_outer: when enabled. remove all geometry on the positive side of the plane
    :type clear_outer: bool
    :param clear_inner: when enabled. remove all geometry on the negative side of the plane
    :type clear_inner: bool
    :rtype: dict
    :return: - geom_cut : output geometry aligned with the plane (new and existing) **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge ) - geom : input and output geometry (result of cut). **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def bmesh_to_mesh(bm: 'bmesh.types.BMesh', mesh: 'bpy.types.Mesh',
                  object: 'bpy.types.Object'):
    ''' BMesh to Mesh. Converts a bmesh to a Mesh. This is reserved for exiting editmode.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param mesh: pointer to a mesh structure to fill in
    :type mesh: 'bpy.types.Mesh'
    :param object: pointer to an object structure
    :type object: 'bpy.types.Object'
    '''

    pass


def bridge_loops(bm: 'bmesh.types.BMesh',
                 edges: typing.List['bmesh.types.BMEdge'], use_pairs: bool,
                 use_cyclic: bool, use_merge: bool, merge_factor: float,
                 twist_offset: int) -> dict:
    ''' Bridge edge loops with faces.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param use_pairs: Undocumented.
    :type use_pairs: bool
    :param use_cyclic: Undocumented.
    :type use_cyclic: bool
    :param use_merge: merge rather than creating faces
    :type use_merge: bool
    :param merge_factor: merge factor
    :type merge_factor: float
    :param twist_offset: twist offset for closed loops
    :type twist_offset: int
    :rtype: dict
    :return: - faces : new faces **type** list of ( bmesh.types.BMFace ) - edges : new edges **type** list of ( bmesh.types.BMEdge )
    '''

    pass


def collapse(bm: 'bmesh.types.BMesh', edges: typing.List['bmesh.types.BMEdge'],
             uvs: bool):
    ''' Collapse Connected. Collapses connected vertices

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param uvs: also collapse UVs and such
    :type uvs: bool
    '''

    pass


def collapse_uvs(bm: 'bmesh.types.BMesh',
                 edges: typing.List['bmesh.types.BMEdge']):
    ''' Collapse Connected UV's. Collapses connected UV vertices.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    '''

    pass


def connect_vert_pair(
        bm: 'bmesh.types.BMesh', verts: typing.List['bmesh.types.BMVert'],
        verts_exclude: typing.List['bmesh.types.BMVert'],
        faces_exclude: typing.List['bmesh.types.BMFace']) -> dict:
    ''' Connect Verts. Split faces by adding edges that connect **verts**.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param verts_exclude: input vertices to explicitly exclude from connecting
    :type verts_exclude: typing.List['bmesh.types.BMVert']
    :param faces_exclude: input faces to explicitly exclude from connecting
    :type faces_exclude: typing.List['bmesh.types.BMFace']
    :rtype: dict
    :return: - edges : **type** list of ( bmesh.types.BMEdge )
    '''

    pass


def connect_verts(bm: 'bmesh.types.BMesh',
                  verts: typing.List['bmesh.types.BMVert'],
                  faces_exclude: typing.List['bmesh.types.BMFace'],
                  check_degenerate: bool) -> dict:
    ''' Connect Verts. Split faces by adding edges that connect **verts**.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param faces_exclude: input faces to explicitly exclude from connecting
    :type faces_exclude: typing.List['bmesh.types.BMFace']
    :param check_degenerate: prevent splits with overlaps & intersections
    :type check_degenerate: bool
    :rtype: dict
    :return: - edges : **type** list of ( bmesh.types.BMEdge )
    '''

    pass


def connect_verts_concave(bm: 'bmesh.types.BMesh',
                          faces: typing.List['bmesh.types.BMFace']) -> dict:
    ''' Connect Verts to form Convex Faces. Ensures all faces are convex **faces**.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :rtype: dict
    :return: - edges : **type** list of ( bmesh.types.BMEdge ) - faces : **type** list of ( bmesh.types.BMFace )
    '''

    pass


def connect_verts_nonplanar(bm: 'bmesh.types.BMesh', angle_limit: float,
                            faces: typing.List['bmesh.types.BMFace']) -> dict:
    ''' Connect Verts Across non Planer Faces. Split faces by connecting edges along non planer **faces**.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param angle_limit: total rotation angle (radians)
    :type angle_limit: float
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :rtype: dict
    :return: - edges : **type** list of ( bmesh.types.BMEdge ) - faces : **type** list of ( bmesh.types.BMFace )
    '''

    pass


def contextual_create(
        bm: 'bmesh.types.BMesh', geom: typing.
        Union[typing.List['bmesh.types.BMEdge'], typing.
              List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']],
        mat_nr: int, use_smooth: bool) -> dict:
    ''' Contextual Create. This is basically F-key, it creates new faces from vertices, makes stuff from edge nets, makes wire edges, etc. It also dissolves faces. Three verts become a triangle, four become a quad. Two become a wire edge.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param geom: input geometry.
    :type geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param mat_nr: material to use
    :type mat_nr: int
    :param use_smooth: smooth to use
    :type use_smooth: bool
    :rtype: dict
    :return: - faces : newly-made face(s) **type** list of ( bmesh.types.BMFace ) - edges : newly-made edge(s) **type** list of ( bmesh.types.BMEdge )
    '''

    pass


def convex_hull(bm: 'bmesh.types.BMesh',
                input: typing.Union[typing.List['bmesh.types.BMEdge'], typing.
                                    List['bmesh.types.BMVert'], typing.
                                    List['bmesh.types.BMFace']],
                use_existing_faces: bool) -> dict:
    ''' Convex Hull Builds a convex hull from the vertices in 'input'. If 'use_existing_faces' is true, the hull will not output triangles that are covered by a pre-existing face. All hull vertices, faces, and edges are added to 'geom.out'. Any input elements that end up inside the hull (i.e. are not used by an output face) are added to the 'interior_geom' slot. The 'unused_geom' slot will contain all interior geometry that is completely unused. Lastly, 'holes_geom' contains edges and faces that were in the input and are part of the hull.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param input: input geometry
    :type input: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param use_existing_faces: skip hull triangles that are covered by a pre-existing face
    :type use_existing_faces: bool
    :rtype: dict
    :return: - geom : **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace ) - geom_interior : **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace ) - geom_unused : **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace ) - geom_holes : **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def create_circle(bm: 'bmesh.types.BMesh', cap_ends: bool, cap_tris: bool,
                  segments: int, radius: float, matrix: 'mathutils.Matrix',
                  calc_uvs: bool) -> dict:
    ''' Creates a Circle.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param cap_ends: whether or not to fill in the ends with faces
    :type cap_ends: bool
    :param cap_tris: fill ends with triangles instead of ngons
    :type cap_tris: bool
    :param segments: number of vertices in the circle
    :type segments: int
    :param radius: Radius of the circle.
    :type radius: float
    :param matrix: matrix to multiply the new geometry with
    :type matrix: 'mathutils.Matrix'
    :param calc_uvs: calculate default UVs
    :type calc_uvs: bool
    :rtype: dict
    :return: - verts : output verts **type** list of ( bmesh.types.BMVert )
    '''

    pass


def create_cone(bm: 'bmesh.types.BMesh', cap_ends: bool, cap_tris: bool,
                segments: int, radius1: float, radius2: float, depth: float,
                matrix: 'mathutils.Matrix', calc_uvs: bool) -> dict:
    ''' Create Cone. Creates a cone with variable depth at both ends

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param cap_ends: whether or not to fill in the ends with faces
    :type cap_ends: bool
    :param cap_tris: fill ends with triangles instead of ngons
    :type cap_tris: bool
    :param segments: number of vertices in the base circle
    :type segments: int
    :param radius1: radius of one end
    :type radius1: float
    :param radius2: radius of the opposite
    :type radius2: float
    :param depth: distance between ends
    :type depth: float
    :param matrix: matrix to multiply the new geometry with
    :type matrix: 'mathutils.Matrix'
    :param calc_uvs: calculate default UVs
    :type calc_uvs: bool
    :rtype: dict
    :return: - verts : output verts **type** list of ( bmesh.types.BMVert )
    '''

    pass


def create_cube(bm: 'bmesh.types.BMesh', size: float,
                matrix: 'mathutils.Matrix', calc_uvs: bool) -> dict:
    ''' Create Cube Creates a cube.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param size: size of the cube
    :type size: float
    :param matrix: matrix to multiply the new geometry with
    :type matrix: 'mathutils.Matrix'
    :param calc_uvs: calculate default UVs
    :type calc_uvs: bool
    :rtype: dict
    :return: - verts : output verts **type** list of ( bmesh.types.BMVert )
    '''

    pass


def create_grid(bm: 'bmesh.types.BMesh', x_segments: int, y_segments: int,
                size: float, matrix: 'mathutils.Matrix',
                calc_uvs: bool) -> dict:
    ''' Create Grid. Creates a grid with a variable number of subdivisions

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param x_segments: number of x segments
    :type x_segments: int
    :param y_segments: number of y segments
    :type y_segments: int
    :param size: size of the grid
    :type size: float
    :param matrix: matrix to multiply the new geometry with
    :type matrix: 'mathutils.Matrix'
    :param calc_uvs: calculate default UVs
    :type calc_uvs: bool
    :rtype: dict
    :return: - verts : output verts **type** list of ( bmesh.types.BMVert )
    '''

    pass


def create_icosphere(bm: 'bmesh.types.BMesh', subdivisions: int, radius: float,
                     matrix: 'mathutils.Matrix', calc_uvs: bool) -> dict:
    ''' Create Ico-Sphere. Creates a grid with a variable number of subdivisions

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param subdivisions: how many times to recursively subdivide the sphere
    :type subdivisions: int
    :param radius: radius
    :type radius: float
    :param matrix: matrix to multiply the new geometry with
    :type matrix: 'mathutils.Matrix'
    :param calc_uvs: calculate default UVs
    :type calc_uvs: bool
    :rtype: dict
    :return: - verts : output verts **type** list of ( bmesh.types.BMVert )
    '''

    pass


def create_monkey(bm: 'bmesh.types.BMesh', matrix: 'mathutils.Matrix',
                  calc_uvs: bool) -> dict:
    ''' Create Suzanne. Creates a monkey (standard blender primitive).

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param matrix: matrix to multiply the new geometry with
    :type matrix: 'mathutils.Matrix'
    :param calc_uvs: calculate default UVs
    :type calc_uvs: bool
    :rtype: dict
    :return: - verts : output verts **type** list of ( bmesh.types.BMVert )
    '''

    pass


def create_uvsphere(bm: 'bmesh.types.BMesh', u_segments: int, v_segments: int,
                    radius: float, matrix: 'mathutils.Matrix',
                    calc_uvs: bool) -> dict:
    ''' Create UV Sphere. Creates a grid with a variable number of subdivisions

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param u_segments: number of u segments
    :type u_segments: int
    :param v_segments: number of v segment
    :type v_segments: int
    :param radius: radius
    :type radius: float
    :param matrix: matrix to multiply the new geometry with
    :type matrix: 'mathutils.Matrix'
    :param calc_uvs: calculate default UVs
    :type calc_uvs: bool
    :rtype: dict
    :return: - verts : output verts **type** list of ( bmesh.types.BMVert )
    '''

    pass


def create_vert(bm: 'bmesh.types.BMesh',
                co: typing.List['mathutils.Vector']) -> dict:
    ''' Make Vertex. Creates a single vertex; this bmop was necessary for click-create-vertex.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param co: the coordinate of the new vert
    :type co: typing.List['mathutils.Vector']
    :rtype: dict
    :return: - vert : the new vert **type** list of ( bmesh.types.BMVert )
    '''

    pass


def delete(bm: 'bmesh.types.BMesh',
           geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.
                              List['bmesh.types.BMVert'], typing.
                              List['bmesh.types.BMFace']],
           context: typing.Union[int, str]):
    ''' Delete Geometry. Utility operator to delete geometry.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param geom: input geometry
    :type geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param context: geometry types to delete
    :type context: typing.Union[int, str]
    '''

    pass


def dissolve_degenerate(bm: 'bmesh.types.BMesh', dist: float,
                        edges: typing.List['bmesh.types.BMEdge']):
    ''' Degenerate Dissolve. Dissolve edges with no length, faces with no area.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param dist: maximum distance to consider degenerate
    :type dist: float
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    '''

    pass


def dissolve_edges(bm: 'bmesh.types.BMesh',
                   edges: typing.List['bmesh.types.BMEdge'], use_verts: bool,
                   use_face_split: bool) -> dict:
    ''' Dissolve Edges.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param use_verts: dissolve verts left between only 2 edges.
    :type use_verts: bool
    :param use_face_split: split off face corners to maintain surrounding geometry
    :type use_face_split: bool
    :rtype: dict
    :return: - region : **type** list of ( bmesh.types.BMFace )
    '''

    pass


def dissolve_faces(bm: 'bmesh.types.BMesh',
                   faces: typing.List['bmesh.types.BMFace'],
                   use_verts: bool) -> dict:
    ''' Dissolve Faces.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param use_verts: dissolve verts left between only 2 edges.
    :type use_verts: bool
    :rtype: dict
    :return: - region : **type** list of ( bmesh.types.BMFace )
    '''

    pass


def dissolve_limit(bm: 'bmesh.types.BMesh', angle_limit: float,
                   use_dissolve_boundaries: bool,
                   verts: typing.List['bmesh.types.BMVert'],
                   edges: typing.List['bmesh.types.BMEdge'],
                   delimit: set) -> dict:
    ''' Limited Dissolve. Dissolve planar faces and co-linear edges.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param angle_limit: total rotation angle (radians)
    :type angle_limit: float
    :param use_dissolve_boundaries: dissolve all vertices in between face boundaries
    :type use_dissolve_boundaries: bool
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param delimit: delimit dissolve operation
    :type delimit: set
    :rtype: dict
    :return: - region : **type** list of ( bmesh.types.BMFace )
    '''

    pass


def dissolve_verts(bm: 'bmesh.types.BMesh',
                   verts: typing.List['bmesh.types.BMVert'],
                   use_face_split: bool, use_boundary_tear: bool):
    ''' Dissolve Verts.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param use_face_split: split off face corners to maintain surrounding geometry
    :type use_face_split: bool
    :param use_boundary_tear: split off face corners instead of merging faces
    :type use_boundary_tear: bool
    '''

    pass


def duplicate(bm: 'bmesh.types.BMesh',
              geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.
                                 List['bmesh.types.BMVert'], typing.
                                 List['bmesh.types.BMFace']],
              dest: 'bmesh.types.BMesh', use_select_history: bool,
              use_edge_flip_from_face: bool) -> dict:
    ''' Duplicate Geometry. Utility operator to duplicate geometry, optionally into a destination mesh.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param geom: input geometry
    :type geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param dest: destination bmesh, if NULL will use current on
    :type dest: 'bmesh.types.BMesh'
    :param use_select_history: Undocumented.
    :type use_select_history: bool
    :param use_edge_flip_from_face: Undocumented.
    :type use_edge_flip_from_face: bool
    :rtype: dict
    :return: - geom_orig : **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace ) - geom : **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace ) - vert_map : **type** dict mapping vert/edge/face types to bmesh.types.BMVert / bmesh.types.BMEdge / bmesh.types.BMFace - edge_map : **type** dict mapping vert/edge/face types to bmesh.types.BMVert / bmesh.types.BMEdge / bmesh.types.BMFace - face_map : **type** dict mapping vert/edge/face types to bmesh.types.BMVert / bmesh.types.BMEdge / bmesh.types.BMFace - boundary_map : **type** dict mapping vert/edge/face types to bmesh.types.BMVert / bmesh.types.BMEdge / bmesh.types.BMFace - isovert_map : **type** dict mapping vert/edge/face types to bmesh.types.BMVert / bmesh.types.BMEdge / bmesh.types.BMFace
    '''

    pass


def edgeloop_fill(bm: 'bmesh.types.BMesh',
                  edges: typing.List['bmesh.types.BMEdge'], mat_nr: int,
                  use_smooth: bool) -> dict:
    ''' Edge Loop Fill. Create faces defined by one or more non overlapping edge loops.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param mat_nr: material to use
    :type mat_nr: int
    :param use_smooth: smooth state to use
    :type use_smooth: bool
    :rtype: dict
    :return: - faces : new faces **type** list of ( bmesh.types.BMFace )
    '''

    pass


def edgenet_fill(bm: 'bmesh.types.BMesh',
                 edges: typing.List['bmesh.types.BMEdge'], mat_nr: int,
                 use_smooth: bool, sides: int) -> dict:
    ''' Edge Net Fill. Create faces defined by enclosed edges.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param mat_nr: material to use
    :type mat_nr: int
    :param use_smooth: smooth state to use
    :type use_smooth: bool
    :param sides: number of sides
    :type sides: int
    :rtype: dict
    :return: - faces : new faces **type** list of ( bmesh.types.BMFace )
    '''

    pass


def edgenet_prepare(bm: 'bmesh.types.BMesh',
                    edges: typing.List['bmesh.types.BMEdge']) -> dict:
    ''' Edge-net Prepare. Identifies several useful edge loop cases and modifies them so they'll become a face when edgenet_fill is called. The cases covered are: - One single loop; an edge is added to connect the ends - Two loops; two edges are added to connect the endpoints (based on the shortest distance between each endpoint).

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :rtype: dict
    :return: - edges : new edges **type** list of ( bmesh.types.BMEdge )
    '''

    pass


def extrude_discrete_faces(
        bm: 'bmesh.types.BMesh', faces: typing.List['bmesh.types.BMFace'],
        use_normal_flip: bool, use_select_history: bool) -> dict:
    ''' Individual Face Extrude. Extrudes faces individually.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param use_normal_flip: Create faces with reversed direction.
    :type use_normal_flip: bool
    :param use_select_history: pass to duplicate
    :type use_select_history: bool
    :rtype: dict
    :return: - faces : output faces **type** list of ( bmesh.types.BMFace )
    '''

    pass


def extrude_edge_only(bm: 'bmesh.types.BMesh',
                      edges: typing.List['bmesh.types.BMEdge'],
                      use_normal_flip: bool, use_select_history: bool) -> dict:
    ''' Extrude Only Edges. Extrudes Edges into faces, note that this is very simple, there's no fancy winged extrusion.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input vertices
    :type edges: typing.List['bmesh.types.BMEdge']
    :param use_normal_flip: Create faces with reversed direction.
    :type use_normal_flip: bool
    :param use_select_history: pass to duplicate
    :type use_select_history: bool
    :rtype: dict
    :return: - geom : output geometry **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def extrude_face_region(
        bm: 'bmesh.types.BMesh', geom: typing.
        Union[typing.List['bmesh.types.BMEdge'], typing.
              List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']],
        edges_exclude: set, use_keep_orig: bool, use_normal_flip: bool,
        use_normal_from_adjacent: bool, use_dissolve_ortho_edges: bool,
        use_select_history: bool) -> dict:
    ''' Extrude Faces. Extrude operator (does not transform)

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param geom: edges and faces
    :type geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param edges_exclude: input edges to explicitly exclude from extrusion
    :type edges_exclude: set
    :param use_keep_orig: keep original geometry (requires geom to include edges).
    :type use_keep_orig: bool
    :param use_normal_flip: Create faces with reversed direction.
    :type use_normal_flip: bool
    :param use_normal_from_adjacent: Use winding from surrounding faces instead of this region.
    :type use_normal_from_adjacent: bool
    :param use_dissolve_ortho_edges: Dissolve edges whose faces form a flat surface.
    :type use_dissolve_ortho_edges: bool
    :param use_select_history: pass to duplicate
    :type use_select_history: bool
    :rtype: dict
    :return: - geom : **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def extrude_vert_indiv(bm: 'bmesh.types.BMesh',
                       verts: typing.List['bmesh.types.BMVert'],
                       use_select_history: bool) -> dict:
    ''' Individual Vertex Extrude. Extrudes wire edges from vertices.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param use_select_history: pass to duplicate
    :type use_select_history: bool
    :rtype: dict
    :return: - edges : output wire edges **type** list of ( bmesh.types.BMEdge ) - verts : output vertices **type** list of ( bmesh.types.BMVert )
    '''

    pass


def face_attribute_fill(bm: 'bmesh.types.BMesh',
                        faces: typing.List['bmesh.types.BMFace'],
                        use_normals: bool, use_data: bool) -> dict:
    ''' Face Attribute Fill. Fill in faces with data from adjacent faces.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param use_normals: copy face winding
    :type use_normals: bool
    :param use_data: copy face data
    :type use_data: bool
    :rtype: dict
    :return: - faces_fail : faces that could not be handled **type** list of ( bmesh.types.BMFace )
    '''

    pass


def find_doubles(
        bm: 'bmesh.types.BMesh', verts: typing.List['bmesh.types.BMVert'],
        keep_verts: typing.List['bmesh.types.BMVert'], dist: float) -> dict:
    ''' Find Doubles. Takes input verts and find vertices they should weld to. Outputs a mapping slot suitable for use with the weld verts bmop. If keep_verts is used, vertices outside that set can only be merged with vertices in that set.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param keep_verts: list of verts to keep
    :type keep_verts: typing.List['bmesh.types.BMVert']
    :param dist: maximum distance
    :type dist: float
    :rtype: dict
    :return: - targetmap : **type** dict mapping vert/edge/face types to bmesh.types.BMVert / bmesh.types.BMEdge / bmesh.types.BMFace
    '''

    pass


def grid_fill(bm: 'bmesh.types.BMesh',
              edges: typing.List['bmesh.types.BMEdge'], mat_nr: int,
              use_smooth: bool, use_interp_simple: bool) -> dict:
    ''' Grid Fill. Create faces defined by 2 disconnected edge loops (which share edges).

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param mat_nr: material to use
    :type mat_nr: int
    :param use_smooth: smooth state to use
    :type use_smooth: bool
    :param use_interp_simple: use simple interpolation
    :type use_interp_simple: bool
    :rtype: dict
    :return: - faces : new faces **type** list of ( bmesh.types.BMFace )
    '''

    pass


def holes_fill(bm: 'bmesh.types.BMesh',
               edges: typing.List['bmesh.types.BMEdge'], sides: int) -> dict:
    ''' Fill Holes. Fill boundary edges with faces, copying surrounding customdata.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param sides: number of face sides to fill
    :type sides: int
    :rtype: dict
    :return: - faces : new faces **type** list of ( bmesh.types.BMFace )
    '''

    pass


def inset_individual(bm: 'bmesh.types.BMesh',
                     faces: typing.List['bmesh.types.BMFace'],
                     thickness: float, depth: float, use_even_offset: bool,
                     use_interpolate: bool, use_relative_offset: bool) -> dict:
    ''' Face Inset (Individual). Insets individual faces.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param thickness: thickness
    :type thickness: float
    :param depth: depth
    :type depth: float
    :param use_even_offset: scale the offset to give more even thickness
    :type use_even_offset: bool
    :param use_interpolate: blend face data across the inset
    :type use_interpolate: bool
    :param use_relative_offset: scale the offset by surrounding geometry
    :type use_relative_offset: bool
    :rtype: dict
    :return: - faces : output faces **type** list of ( bmesh.types.BMFace )
    '''

    pass


def inset_region(
        bm: 'bmesh.types.BMesh', faces: typing.List['bmesh.types.BMFace'],
        faces_exclude: typing.List['bmesh.types.BMFace'], use_boundary: bool,
        use_even_offset: bool, use_interpolate: bool,
        use_relative_offset: bool, use_edge_rail: bool, thickness: float,
        depth: float, use_outset: bool) -> dict:
    ''' Face Inset (Regions). Inset or outset face regions.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param faces_exclude: input faces to explicitly exclude from inset
    :type faces_exclude: typing.List['bmesh.types.BMFace']
    :param use_boundary: inset face boundaries
    :type use_boundary: bool
    :param use_even_offset: scale the offset to give more even thickness
    :type use_even_offset: bool
    :param use_interpolate: blend face data across the inset
    :type use_interpolate: bool
    :param use_relative_offset: scale the offset by surrounding geometry
    :type use_relative_offset: bool
    :param use_edge_rail: inset the region along existing edges
    :type use_edge_rail: bool
    :param thickness: thickness
    :type thickness: float
    :param depth: depth
    :type depth: float
    :param use_outset: outset rather than inset
    :type use_outset: bool
    :rtype: dict
    :return: - faces : output faces **type** list of ( bmesh.types.BMFace )
    '''

    pass


def join_triangles(bm: 'bmesh.types.BMesh',
                   faces: typing.List['bmesh.types.BMFace'], cmp_seam: bool,
                   cmp_sharp: bool, cmp_uvs: bool, cmp_vcols: bool,
                   cmp_materials: bool, angle_face_threshold: float,
                   angle_shape_threshold: float) -> dict:
    ''' Join Triangles. Tries to intelligently join triangles according to angle threshold and delimiters.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input geometry.
    :type faces: typing.List['bmesh.types.BMFace']
    :param cmp_seam: Compare seam
    :type cmp_seam: bool
    :param cmp_sharp: Compare sharp
    :type cmp_sharp: bool
    :param cmp_uvs: Compare UVs
    :type cmp_uvs: bool
    :param cmp_vcols: compare VCols
    :type cmp_vcols: bool
    :param cmp_materials: compare materials
    :type cmp_materials: bool
    :param angle_face_threshold: Undocumented.
    :type angle_face_threshold: float
    :param angle_shape_threshold: Undocumented.
    :type angle_shape_threshold: float
    :rtype: dict
    :return: - faces : joined faces **type** list of ( bmesh.types.BMFace )
    '''

    pass


def mesh_to_bmesh(bm: 'bmesh.types.BMesh', mesh: 'bpy.types.Mesh',
                  object: 'bpy.types.Object', use_shapekey: bool):
    ''' Mesh to BMesh. Load the contents of a mesh into the bmesh. this bmop is private, it's reserved exclusively for entering editmode.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param mesh: pointer to a Mesh structure
    :type mesh: 'bpy.types.Mesh'
    :param object: pointer to an Object structure
    :type object: 'bpy.types.Object'
    :param use_shapekey: load active shapekey coordinates into verts
    :type use_shapekey: bool
    '''

    pass


def mirror(bm: 'bmesh.types.BMesh',
           geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.
                              List['bmesh.types.BMVert'], typing.
                              List['bmesh.types.BMFace']],
           matrix: 'mathutils.Matrix', merge_dist: float,
           axis: typing.Union[int, str], mirror_u: bool, mirror_v: bool,
           mirror_udim: bool, use_shapekey: bool) -> dict:
    ''' Mirror. Mirrors geometry along an axis. The resulting geometry is welded on using merge_dist. Pairs of original/mirrored vertices are welded using the merge_dist parameter (which defines the minimum distance for welding to happen).

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param geom: input geometry
    :type geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param matrix: matrix defining the mirror transformation
    :type matrix: 'mathutils.Matrix'
    :param merge_dist: maximum distance for merging. does no merging if 0.
    :type merge_dist: float
    :param axis: the axis to use.
    :type axis: typing.Union[int, str]
    :param mirror_u: mirror UVs across the u axis
    :type mirror_u: bool
    :param mirror_v: mirror UVs across the v axis
    :type mirror_v: bool
    :param mirror_udim: mirror UVs in each tile
    :type mirror_udim: bool
    :param use_shapekey: Transform shape keys too.
    :type use_shapekey: bool
    :rtype: dict
    :return: - geom : output geometry, mirrored **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def object_load_bmesh(bm: 'bmesh.types.BMesh', scene: 'bpy.types.Scene',
                      object: 'bpy.types.Object'):
    ''' Object Load BMesh. Loads a bmesh into an object/mesh. This is a "private" bmop.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param scene: pointer to an scene structure
    :type scene: 'bpy.types.Scene'
    :param object: pointer to an object structure
    :type object: 'bpy.types.Object'
    '''

    pass


def offset_edgeloops(bm: 'bmesh.types.BMesh',
                     edges: typing.List['bmesh.types.BMEdge'],
                     use_cap_endpoint: bool) -> dict:
    ''' Edge-loop Offset. Creates edge loops based on simple edge-outset method.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param use_cap_endpoint: extend loop around end-points
    :type use_cap_endpoint: bool
    :rtype: dict
    :return: - edges : output edges **type** list of ( bmesh.types.BMEdge )
    '''

    pass


def planar_faces(bm: 'bmesh.types.BMesh',
                 faces: typing.List['bmesh.types.BMFace'], iterations: int,
                 factor: float) -> dict:
    ''' Planar Faces. Iteratively flatten faces.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input geometry.
    :type faces: typing.List['bmesh.types.BMFace']
    :param iterations: Number of times to flatten faces (for when connected faces are used)
    :type iterations: int
    :param factor: Influence for making planar each iteration
    :type factor: float
    :rtype: dict
    :return: - geom : output slot, computed boundary geometry. **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def pointmerge(bm: 'bmesh.types.BMesh',
               verts: typing.List['bmesh.types.BMVert'],
               merge_co: typing.List['mathutils.Vector']):
    ''' Point Merge. Merge verts together at a point.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param verts: input vertices (all verts will be merged into the first).
    :type verts: typing.List['bmesh.types.BMVert']
    :param merge_co: Position to merge at.
    :type merge_co: typing.List['mathutils.Vector']
    '''

    pass


def pointmerge_facedata(bm: 'bmesh.types.BMesh',
                        verts: typing.List['bmesh.types.BMVert'],
                        vert_snap: 'bmesh.types.BMVert'):
    ''' Face-Data Point Merge. Merge uv/vcols at a specific vertex.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param vert_snap: snap vertex
    :type vert_snap: 'bmesh.types.BMVert'
    '''

    pass


def poke(bm: 'bmesh.types.BMesh', faces: typing.List['bmesh.types.BMFace'],
         offset: float, center_mode: typing.Union[int, str],
         use_relative_offset: bool) -> dict:
    ''' Pokes a face. Splits a face into a triangle fan.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param offset: center vertex offset along normal
    :type offset: float
    :param center_mode: calculation mode for center vertex
    :type center_mode: typing.Union[int, str]
    :param use_relative_offset: apply offset
    :type use_relative_offset: bool
    :rtype: dict
    :return: - verts : output verts **type** list of ( bmesh.types.BMVert ) - faces : output faces **type** list of ( bmesh.types.BMFace )
    '''

    pass


def recalc_face_normals(bm: 'bmesh.types.BMesh',
                        faces: typing.List['bmesh.types.BMFace']):
    ''' Right-Hand Faces. Computes an "outside" normal for the specified input faces.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    '''

    pass


def region_extend(
        bm: 'bmesh.types.BMesh', geom: typing.
        Union[typing.List['bmesh.types.BMEdge'], typing.
              List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']],
        use_contract: bool, use_faces: bool, use_face_step: bool) -> dict:
    ''' Region Extend. used to implement the select more/less tools. this puts some geometry surrounding regions of geometry in geom into geom.out. if use_faces is 0 then geom.out spits out verts and edges, otherwise it spits out faces.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param geom: input geometry
    :type geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param use_contract: find boundary inside the regions, not outside.
    :type use_contract: bool
    :param use_faces: extend from faces instead of edges
    :type use_faces: bool
    :param use_face_step: step over connected faces
    :type use_face_step: bool
    :rtype: dict
    :return: - geom : output slot, computed boundary geometry. **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def remove_doubles(bm: 'bmesh.types.BMesh',
                   verts: typing.List['bmesh.types.BMVert'], dist: float):
    ''' Remove Doubles. Finds groups of vertices closer than dist and merges them together, using the weld verts bmop.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param verts: input verts
    :type verts: typing.List['bmesh.types.BMVert']
    :param dist: minimum distance
    :type dist: float
    '''

    pass


def reverse_colors(bm: 'bmesh.types.BMesh',
                   faces: typing.List['bmesh.types.BMFace'], color_index: int):
    ''' Color Reverse Reverse the loop colors.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param color_index: index into color attribute list
    :type color_index: int
    '''

    pass


def reverse_faces(bm: 'bmesh.types.BMesh',
                  faces: typing.List['bmesh.types.BMFace'],
                  flip_multires: bool):
    ''' Reverse Faces. Reverses the winding (vertex order) of faces. This has the effect of flipping the normal.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param flip_multires: maintain multi-res offset
    :type flip_multires: bool
    '''

    pass


def reverse_uvs(bm: 'bmesh.types.BMesh',
                faces: typing.List['bmesh.types.BMFace']):
    ''' UV Reverse. Reverse the UV's

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    '''

    pass


def rotate(bm: 'bmesh.types.BMesh', cent: typing.List['mathutils.Vector'],
           matrix: 'mathutils.Matrix',
           verts: typing.List['bmesh.types.BMVert'], space: 'mathutils.Matrix',
           use_shapekey: bool):
    ''' Rotate. Rotate vertices around a center, using a 3x3 rotation matrix.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param cent: center of rotation
    :type cent: typing.List['mathutils.Vector']
    :param matrix: matrix defining rotation
    :type matrix: 'mathutils.Matrix'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param space: matrix to define the space (typically object matrix)
    :type space: 'mathutils.Matrix'
    :param use_shapekey: Transform shape keys too.
    :type use_shapekey: bool
    '''

    pass


def rotate_colors(bm: 'bmesh.types.BMesh',
                  faces: typing.List['bmesh.types.BMFace'], use_ccw: bool,
                  color_index: int):
    ''' Color Rotation. Cycle the loop colors

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param use_ccw: rotate counter-clockwise if true, otherwise clockwise
    :type use_ccw: bool
    :param color_index: index into color attribute list
    :type color_index: int
    '''

    pass


def rotate_edges(bm: 'bmesh.types.BMesh',
                 edges: typing.List['bmesh.types.BMEdge'],
                 use_ccw: bool) -> dict:
    ''' Edge Rotate. Rotates edges topologically. Also known as "spin edge" to some people. Simple example: [/] becomes [|] then [\] .

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param use_ccw: rotate edge counter-clockwise if true, otherwise clockwise
    :type use_ccw: bool
    :rtype: dict
    :return: - edges : newly spun edges **type** list of ( bmesh.types.BMEdge )
    '''

    pass


def rotate_uvs(bm: 'bmesh.types.BMesh',
               faces: typing.List['bmesh.types.BMFace'], use_ccw: bool):
    ''' UV Rotation. Cycle the loop UV's

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param use_ccw: rotate counter-clockwise if true, otherwise clockwise
    :type use_ccw: bool
    '''

    pass


def scale(bm: 'bmesh.types.BMesh', vec: typing.List['mathutils.Vector'],
          space: 'mathutils.Matrix', verts: typing.List['bmesh.types.BMVert'],
          use_shapekey: bool):
    ''' Scale. Scales vertices by an offset.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param vec: scale factor
    :type vec: typing.List['mathutils.Vector']
    :param space: matrix to define the space (typically object matrix)
    :type space: 'mathutils.Matrix'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param use_shapekey: Transform shape keys too.
    :type use_shapekey: bool
    '''

    pass


def smooth_laplacian_vert(
        bm: 'bmesh.types.BMesh', verts: typing.List['bmesh.types.BMVert'],
        lambda_factor: float, lambda_border: float, use_x: bool, use_y: bool,
        use_z: bool, preserve_volume: bool):
    ''' Vertex Smooth Laplacian. Smooths vertices by using Laplacian smoothing propose by. Desbrun, et al. Implicit Fairing of Irregular Meshes using Diffusion and Curvature Flow.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param lambda_factor: lambda param
    :type lambda_factor: float
    :param lambda_border: lambda param in border
    :type lambda_border: float
    :param use_x: Smooth object along X axis
    :type use_x: bool
    :param use_y: Smooth object along Y axis
    :type use_y: bool
    :param use_z: Smooth object along Z axis
    :type use_z: bool
    :param preserve_volume: Apply volume preservation after smooth
    :type preserve_volume: bool
    '''

    pass


def smooth_vert(bm: 'bmesh.types.BMesh',
                verts: typing.List['bmesh.types.BMVert'], factor: float,
                mirror_clip_x: bool, mirror_clip_y: bool, mirror_clip_z: bool,
                clip_dist: float, use_axis_x: bool, use_axis_y: bool,
                use_axis_z: bool):
    ''' Vertex Smooth. Smooths vertices by using a basic vertex averaging scheme.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param factor: smoothing factor
    :type factor: float
    :param mirror_clip_x: set vertices close to the x axis before the operation to 0
    :type mirror_clip_x: bool
    :param mirror_clip_y: set vertices close to the y axis before the operation to 0
    :type mirror_clip_y: bool
    :param mirror_clip_z: set vertices close to the z axis before the operation to 0
    :type mirror_clip_z: bool
    :param clip_dist: clipping threshold for the above three slots
    :type clip_dist: float
    :param use_axis_x: smooth vertices along X axis
    :type use_axis_x: bool
    :param use_axis_y: smooth vertices along Y axis
    :type use_axis_y: bool
    :param use_axis_z: smooth vertices along Z axis
    :type use_axis_z: bool
    '''

    pass


def solidify(bm: 'bmesh.types.BMesh',
             geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.
                                List['bmesh.types.BMVert'], typing.
                                List['bmesh.types.BMFace']],
             thickness: float) -> dict:
    ''' Solidify. Turns a mesh into a shell with thickness

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param geom: input geometry
    :type geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param thickness: thickness
    :type thickness: float
    :rtype: dict
    :return: - geom : **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def spin(bm: 'bmesh.types.BMesh', geom: typing.
         Union[typing.List['bmesh.types.BMEdge'], typing.
               List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']],
         cent: typing.List['mathutils.Vector'],
         axis: typing.List['mathutils.Vector'],
         dvec: typing.List['mathutils.Vector'], angle: float,
         space: 'mathutils.Matrix', steps: int, use_merge: bool,
         use_normal_flip: bool, use_duplicate: bool) -> dict:
    ''' Spin. Extrude or duplicate geometry a number of times, rotating and possibly translating after each step

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param geom: input geometry
    :type geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param cent: rotation center
    :type cent: typing.List['mathutils.Vector']
    :param axis: rotation axis
    :type axis: typing.List['mathutils.Vector']
    :param dvec: translation delta per step
    :type dvec: typing.List['mathutils.Vector']
    :param angle: total rotation angle (radians)
    :type angle: float
    :param space: matrix to define the space (typically object matrix)
    :type space: 'mathutils.Matrix'
    :param steps: number of steps
    :type steps: int
    :param use_merge: Merge first/last when the angle is a full revolution.
    :type use_merge: bool
    :param use_normal_flip: Create faces with reversed direction.
    :type use_normal_flip: bool
    :param use_duplicate: duplicate or extrude?
    :type use_duplicate: bool
    :rtype: dict
    :return: - geom_last : result of last step **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def split(bm: 'bmesh.types.BMesh', geom: typing.
          Union[typing.List['bmesh.types.BMEdge'], typing.
                List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']],
          dest: 'bmesh.types.BMesh', use_only_faces: bool) -> dict:
    ''' Split Off Geometry. Disconnect geometry from adjacent edges and faces, optionally into a destination mesh.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param geom: input geometry
    :type geom: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param dest: destination bmesh, if NULL will use current one
    :type dest: 'bmesh.types.BMesh'
    :param use_only_faces: when enabled. don't duplicate loose verts/edges
    :type use_only_faces: bool
    :rtype: dict
    :return: - geom : **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace ) - boundary_map : **type** dict mapping vert/edge/face types to bmesh.types.BMVert / bmesh.types.BMEdge / bmesh.types.BMFace - isovert_map : **type** dict mapping vert/edge/face types to bmesh.types.BMVert / bmesh.types.BMEdge / bmesh.types.BMFace
    '''

    pass


def split_edges(
        bm: 'bmesh.types.BMesh', edges: typing.List['bmesh.types.BMEdge'],
        verts: typing.List['bmesh.types.BMVert'], use_verts: bool) -> dict:
    ''' Edge Split. Disconnects faces along input edges.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param verts: optional tag verts, use to have greater control of splits
    :type verts: typing.List['bmesh.types.BMVert']
    :param use_verts: use 'verts' for splitting, else just find verts to split from edges
    :type use_verts: bool
    :rtype: dict
    :return: - edges : old output disconnected edges **type** list of ( bmesh.types.BMEdge )
    '''

    pass


def subdivide_edgering(bm: 'bmesh.types.BMesh',
                       edges: typing.List['bmesh.types.BMEdge'],
                       interp_mode: typing.Union[int, str], smooth: float,
                       cuts: int, profile_shape: typing.Union[int, str],
                       profile_shape_factor: float) -> dict:
    ''' Subdivide Edge-Ring. Take an edge-ring, and subdivide with interpolation options.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input vertices
    :type edges: typing.List['bmesh.types.BMEdge']
    :param interp_mode: interpolation method
    :type interp_mode: typing.Union[int, str]
    :param smooth: smoothness factor
    :type smooth: float
    :param cuts: number of cuts
    :type cuts: int
    :param profile_shape: profile shape type
    :type profile_shape: typing.Union[int, str]
    :param profile_shape_factor: how much intermediary new edges are shrunk/expanded
    :type profile_shape_factor: float
    :rtype: dict
    :return: - faces : output faces **type** list of ( bmesh.types.BMFace )
    '''

    pass


def subdivide_edges(
        bm: 'bmesh.types.BMesh', edges: typing.List['bmesh.types.BMEdge'],
        smooth: float, smooth_falloff: typing.Union[int, str], fractal: float,
        along_normal: float, cuts: int, seed: int, custom_patterns: dict,
        edge_percents: dict, quad_corner_type: typing.Union[int, str],
        use_grid_fill: bool, use_single_edge: bool, use_only_quads: bool,
        use_sphere: bool, use_smooth_even: bool) -> dict:
    ''' Subdivide Edges. Advanced operator for subdividing edges with options for face patterns, smoothing and randomization.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param smooth: smoothness factor
    :type smooth: float
    :param smooth_falloff: smooth falloff type
    :type smooth_falloff: typing.Union[int, str]
    :param fractal: fractal randomness factor
    :type fractal: float
    :param along_normal: apply fractal displacement along normal only
    :type along_normal: float
    :param cuts: number of cuts
    :type cuts: int
    :param seed: seed for the random number generator
    :type seed: int
    :param custom_patterns: uses custom pointers
    :type custom_patterns: dict
    :param edge_percents: Undocumented.
    :type edge_percents: dict
    :param quad_corner_type: quad corner type
    :type quad_corner_type: typing.Union[int, str]
    :param use_grid_fill: fill in fully-selected faces with a grid
    :type use_grid_fill: bool
    :param use_single_edge: tessellate the case of one edge selected in a quad or triangle
    :type use_single_edge: bool
    :param use_only_quads: Only subdivide quads (for loop-cut).
    :type use_only_quads: bool
    :param use_sphere: for making new primitives only
    :type use_sphere: bool
    :param use_smooth_even: maintain even offset when smoothing
    :type use_smooth_even: bool
    :rtype: dict
    :return: - geom_inner : **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace ) - geom_split : **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace ) - geom : contains all output geometry **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def symmetrize(bm: 'bmesh.types.BMesh',
               input: typing.Union[typing.List['bmesh.types.BMEdge'], typing.
                                   List['bmesh.types.BMVert'], typing.
                                   List['bmesh.types.BMFace']],
               direction: typing.Union[int, str], dist: float,
               use_shapekey: bool) -> dict:
    ''' Symmetrize. Makes the mesh elements in the "input" slot symmetrical. Unlike normal mirroring, it only copies in one direction, as specified by the "direction" slot. The edges and faces that cross the plane of symmetry are split as needed to enforce symmetry. All new vertices, edges, and faces are added to the "geom.out" slot.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param input: input geometry
    :type input: typing.Union[typing.List['bmesh.types.BMEdge'], typing.List['bmesh.types.BMVert'], typing.List['bmesh.types.BMFace']]
    :param direction: axis to use
    :type direction: typing.Union[int, str]
    :param dist: minimum distance
    :type dist: float
    :param use_shapekey: Transform shape keys too.
    :type use_shapekey: bool
    :rtype: dict
    :return: - geom : **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def transform(bm: 'bmesh.types.BMesh', matrix: 'mathutils.Matrix',
              space: 'mathutils.Matrix',
              verts: typing.List['bmesh.types.BMVert'], use_shapekey: bool):
    ''' Transform. Transforms a set of vertices by a matrix. Multiplies the vertex coordinates with the matrix.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param matrix: transform matrix
    :type matrix: 'mathutils.Matrix'
    :param space: matrix to define the space (typically object matrix)
    :type space: 'mathutils.Matrix'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param use_shapekey: Transform shape keys too.
    :type use_shapekey: bool
    '''

    pass


def translate(bm: 'bmesh.types.BMesh', vec: typing.List['mathutils.Vector'],
              space: 'mathutils.Matrix',
              verts: typing.List['bmesh.types.BMVert'], use_shapekey: bool):
    ''' Translate. Translate vertices by an offset.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param vec: translation offset
    :type vec: typing.List['mathutils.Vector']
    :param space: matrix to define the space (typically object matrix)
    :type space: 'mathutils.Matrix'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param use_shapekey: Transform shape keys too.
    :type use_shapekey: bool
    '''

    pass


def triangle_fill(bm: 'bmesh.types.BMesh', use_beauty: bool,
                  use_dissolve: bool, edges: typing.List['bmesh.types.BMEdge'],
                  normal: typing.List['mathutils.Vector']) -> dict:
    ''' Triangle Fill. Fill edges with triangles

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param use_beauty: use best triangulation division
    :type use_beauty: bool
    :param use_dissolve: dissolve resulting faces
    :type use_dissolve: bool
    :param edges: input edges
    :type edges: typing.List['bmesh.types.BMEdge']
    :param normal: optionally pass the fill normal to use
    :type normal: typing.List['mathutils.Vector']
    :rtype: dict
    :return: - geom : new faces and edges **type** list of ( bmesh.types.BMVert , bmesh.types.BMEdge , bmesh.types.BMFace )
    '''

    pass


def triangulate(bm: 'bmesh.types.BMesh',
                faces: typing.List['bmesh.types.BMFace'],
                quad_method: typing.Union[int, str],
                ngon_method: typing.Union[int, str]) -> dict:
    ''' Triangulate.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param quad_method: method for splitting the quads into triangles
    :type quad_method: typing.Union[int, str]
    :param ngon_method: method for splitting the polygons into triangles
    :type ngon_method: typing.Union[int, str]
    :rtype: dict
    :return: - edges : **type** list of ( bmesh.types.BMEdge ) - faces : **type** list of ( bmesh.types.BMFace ) - face_map : **type** dict mapping vert/edge/face types to bmesh.types.BMVert / bmesh.types.BMEdge / bmesh.types.BMFace - face_map_double : duplicate faces **type** dict mapping vert/edge/face types to bmesh.types.BMVert / bmesh.types.BMEdge / bmesh.types.BMFace
    '''

    pass


def unsubdivide(bm: 'bmesh.types.BMesh',
                verts: typing.List['bmesh.types.BMVert'], iterations: int):
    ''' Un-Subdivide. Reduce detail in geometry containing grids.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param verts: input vertices
    :type verts: typing.List['bmesh.types.BMVert']
    :param iterations: number of times to unsubdivide
    :type iterations: int
    '''

    pass


def weld_verts(bm: 'bmesh.types.BMesh',
               targetmap: typing.Union[dict, dict, dict]):
    ''' Weld Verts. Welds verts together (kind-of like remove doubles, merge, etc, all of which use or will use this bmop). You pass in mappings from vertices to the vertices they weld with.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param targetmap: maps welded vertices to verts they should weld to
    :type targetmap: typing.Union[dict, dict, dict]
    '''

    pass


def wireframe(bm: 'bmesh.types.BMesh',
              faces: typing.List['bmesh.types.BMFace'], thickness: float,
              offset: float, use_replace: bool, use_boundary: bool,
              use_even_offset: bool, use_crease: bool, crease_weight: float,
              use_relative_offset: bool, material_offset: int) -> dict:
    ''' Wire Frame. Makes a wire-frame copy of faces.

    :param bm: The bmesh to operate on.
    :type bm: 'bmesh.types.BMesh'
    :param faces: input faces
    :type faces: typing.List['bmesh.types.BMFace']
    :param thickness: thickness
    :type thickness: float
    :param offset: offset the thickness from the center
    :type offset: float
    :param use_replace: remove original geometry
    :type use_replace: bool
    :param use_boundary: inset face boundaries
    :type use_boundary: bool
    :param use_even_offset: scale the offset to give more even thickness
    :type use_even_offset: bool
    :param use_crease: crease hub edges for improved subdivision surface
    :type use_crease: bool
    :param crease_weight: the mean crease weight for resulting edges
    :type crease_weight: float
    :param use_relative_offset: scale the offset by surrounding geometry
    :type use_relative_offset: bool
    :param material_offset: offset material index of generated faces
    :type material_offset: int
    :rtype: dict
    :return: - faces : output faces **type** list of ( bmesh.types.BMFace )
    '''

    pass



import sys
import typing


def action_sanitize():
    ''' Deprecated, will be removed in Blender 3.3. Make action suitable for use as a Legacy Pose Library

    '''

    pass


def apply_pose(pose_index: int = -1):
    ''' Deprecated, will be removed in Blender 3.3. Apply specified Legacy Pose Library pose to the rig

    :param pose_index: Pose, Index of the pose to apply (-2 for no change to pose, -1 for poselib active pose)
    :type pose_index: int
    '''

    pass


def apply_pose_asset(blend_factor: float = 1.0, flipped: bool = False):
    ''' Apply the given Pose Action to the rig

    :param blend_factor: Blend Factor, Amount that the pose is applied on top of the existing poses
    :type blend_factor: float
    :param flipped: Apply Flipped, When enabled, applies the pose flipped over the X-axis
    :type flipped: bool
    '''

    pass


def apply_pose_asset_for_keymap():
    ''' Apply the given Pose Action to the rig :file: addons/pose_library/operators.py\:431 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$431> _

    '''

    pass


def blend_pose_asset(blend_factor: float = 0.0,
                     flipped: bool = False,
                     release_confirm: bool = False):
    ''' Blend the given Pose Action to the rig

    :param blend_factor: Blend Factor, Amount that the pose is applied on top of the existing poses
    :type blend_factor: float
    :param flipped: Apply Flipped, When enabled, applies the pose flipped over the X-axis
    :type flipped: bool
    :param release_confirm: Confirm on Release, Always confirm operation when releasing button
    :type release_confirm: bool
    '''

    pass


def blend_pose_asset_for_keymap():
    ''' Blend the given Pose Action to the rig :file: addons/pose_library/operators.py\:403 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$403> _

    '''

    pass


def browse_interactive(pose_index: int = -1):
    ''' Deprecated, will be removed in Blender 3.3. Interactively browse Legacy Pose Library poses in 3D-View

    :param pose_index: Pose, Index of the pose to apply (-2 for no change to pose, -1 for poselib active pose)
    :type pose_index: int
    '''

    pass


def convert_old_object_poselib():
    ''' Create a pose asset for each pose marker in this legacy pose library data-block :file: addons/pose_library/operators.py\:487 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$487> _

    '''

    pass


def convert_old_poselib():
    ''' Create a pose asset for each pose marker in the current action :file: addons/pose_library/operators.py\:453 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$453> _

    '''

    pass


def copy_as_asset():
    ''' Create a new pose asset on the clipboard, to be pasted into an Asset Browser :file: addons/pose_library/operators.py\:210 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$210> _

    '''

    pass


def create_pose_asset(pose_name: str = "", activate_new_action: bool = True):
    ''' Create a new Action that contains the pose of the selected bones, and mark it as Asset. The asset will be stored in the current blend file

    :param pose_name: Pose Name
    :type pose_name: str
    :param activate_new_action: Activate New Action
    :type activate_new_action: bool
    '''

    pass


def new():
    ''' Deprecated, will be removed in Blender 3.3. Add New Legacy Pose Library to active Object

    '''

    pass


def paste_asset():
    ''' Paste the Asset that was previously copied using Copy As Asset :file: addons/pose_library/operators.py\:285 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$285> _

    '''

    pass


def pose_add(frame: int = 1, name: str = "Pose"):
    ''' Deprecated, will be removed in Blender 3.3. Add the current Pose to the active Legacy Pose Library

    :param frame: Frame, Frame to store pose on
    :type frame: int
    :param name: Pose Name, Name of newly added Pose
    :type name: str
    '''

    pass


def pose_asset_select_bones(select: bool = True, flipped: bool = False):
    ''' Select those bones that are used in this pose

    :param select: Select
    :type select: bool
    :param flipped: Flipped
    :type flipped: bool
    '''

    pass


def pose_move(pose: typing.Union[int, str] = '',
              direction: typing.Union[int, str] = 'UP'):
    ''' Deprecated, will be removed in Blender 3.3. Move the pose up or down in the active Legacy Pose Library

    :param pose: Pose, The pose to move
    :type pose: typing.Union[int, str]
    :param direction: Direction, Direction to move the chosen pose towards
    :type direction: typing.Union[int, str]
    '''

    pass


def pose_remove(pose: typing.Union[int, str] = ''):
    ''' Deprecated, will be removed in Blender 3.3. Remove nth pose from the active Legacy Pose Library

    :param pose: Pose, The pose to remove
    :type pose: typing.Union[int, str]
    '''

    pass


def pose_rename(name: str = "RenamedPose", pose: typing.Union[int, str] = ''):
    ''' Deprecated, will be removed in Blender 3.3. Rename specified pose from the active Legacy Pose Library

    :param name: New Pose Name, New name for pose
    :type name: str
    :param pose: Pose, The pose to rename
    :type pose: typing.Union[int, str]
    '''

    pass


def restore_previous_action():
    ''' Switch back to the previous Action, after creating a pose asset :file: addons/pose_library/operators.py\:163 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$163> _

    '''

    pass


def unlink():
    ''' Deprecated, will be removed in Blender 3.3. Remove Legacy Pose Library from active Object

    '''

    pass



import sys
import typing
import bpy.types

active_annotation_layer: 'bpy.types.GPencilLayer' = None

active_bone: 'bpy.types.EditBone' = None

active_editable_fcurve: 'bpy.types.FCurve' = None

active_file: 'bpy.types.FileSelectEntry' = None

active_gpencil_frame: list = None

active_gpencil_layer: typing.List['bpy.types.GPencilLayer'] = None

active_nla_strip: 'bpy.types.NlaStrip' = None

active_nla_track: 'bpy.types.NlaTrack' = None

active_node: 'bpy.types.Node' = None

active_object: 'bpy.types.Object' = None

active_operator: 'bpy.types.Operator' = None

active_pose_bone: 'bpy.types.PoseBone' = None

active_sequence_strip: 'bpy.types.Sequence' = None

annotation_data: 'bpy.types.GreasePencil' = None

annotation_data_owner: 'bpy.types.ID' = None

area: 'bpy.types.Area' = None

armature: 'bpy.types.Armature' = None

asset_file_handle: 'bpy.types.FileSelectEntry' = None
''' The file of an active asset. Avoid using this, it will be replaced by a proper AssetHandle design
'''

asset_library_ref: 'bpy.types.AssetLibraryReference' = None

blend_data: 'bpy.types.BlendData' = None

bone: 'bpy.types.Bone' = None

brush: 'bpy.types.Brush' = None

camera: 'bpy.types.Camera' = None

cloth: 'bpy.types.ClothModifier' = None

collection: 'bpy.types.Collection' = None

collision: 'bpy.types.CollisionModifier' = None

curve: 'bpy.types.Curve' = None

curves: 'bpy.types.Curves' = None

dynamic_paint: 'bpy.types.DynamicPaintModifier' = None

edit_bone: 'bpy.types.EditBone' = None

edit_image: 'bpy.types.Image' = None

edit_mask: 'bpy.types.Mask' = None

edit_movieclip: 'bpy.types.MovieClip' = None

edit_object: 'bpy.types.Object' = None

edit_text: 'bpy.types.Text' = None

editable_bones: typing.List['bpy.types.EditBone'] = None

editable_fcurves: typing.List['bpy.types.FCurve'] = None

editable_gpencil_layers: typing.List['bpy.types.GPencilLayer'] = None

editable_gpencil_strokes: typing.List['bpy.types.GPencilStroke'] = None

editable_objects: typing.List['bpy.types.Object'] = None

engine: str = None

fluid = None

gizmo_group: 'bpy.types.GizmoGroup' = None

gpencil: 'bpy.types.GreasePencil' = None

gpencil_data: 'bpy.types.GreasePencil' = None

gpencil_data_owner: 'bpy.types.ID' = None

id: 'bpy.types.ID' = None

image_paint_object: 'bpy.types.Object' = None

lattice: 'bpy.types.Lattice' = None

layer_collection: 'bpy.types.LayerCollection' = None

light: 'bpy.types.Light' = None

lightprobe: 'bpy.types.LightProbe' = None

line_style: 'bpy.types.FreestyleLineStyle' = None

material: 'bpy.types.Material' = None

material_slot: 'bpy.types.MaterialSlot' = None

mesh: 'bpy.types.Mesh' = None

meta_ball: 'bpy.types.MetaBall' = None

mode: typing.Union[int, str] = None

object: 'bpy.types.Object' = None

objects_in_mode: typing.List['bpy.types.Object'] = None

objects_in_mode_unique_data: typing.List['bpy.types.Object'] = None

particle_edit_object: 'bpy.types.Object' = None

particle_settings: 'bpy.types.ParticleSettings' = None

particle_system: 'bpy.types.ParticleSystem' = None

particle_system_editable: 'bpy.types.ParticleSystem' = None

pointcloud: 'bpy.types.PointCloud' = None

pose_bone: 'bpy.types.PoseBone' = None

pose_object: 'bpy.types.Object' = None

preferences: 'bpy.types.Preferences' = None

region: 'bpy.types.Region' = None

region_data: 'bpy.types.RegionView3D' = None

scene: 'bpy.types.Scene' = None

screen: 'bpy.types.Screen' = None

sculpt_object: 'bpy.types.Object' = None

selectable_objects: typing.List['bpy.types.Object'] = None

selected_asset_files: typing.List['bpy.types.FileSelectEntry'] = None

selected_bones: typing.List['bpy.types.EditBone'] = None

selected_editable_actions: typing.List['bpy.types.Action'] = None

selected_editable_bones: typing.List['bpy.types.EditBone'] = None

selected_editable_fcurves: typing.List['bpy.types.FCurve'] = None

selected_editable_keyframes: typing.List['bpy.types.Keyframe'] = None

selected_editable_objects: typing.List['bpy.types.Object'] = None

selected_editable_sequences: typing.List['bpy.types.Sequence'] = None

selected_files: typing.List['bpy.types.FileSelectEntry'] = None

selected_ids: typing.List['bpy.types.ID'] = None

selected_movieclip_tracks: typing.List['bpy.types.MovieTrackingTrack'] = None

selected_nla_strips: typing.List['bpy.types.NlaStrip'] = None

selected_nodes: typing.List['bpy.types.Node'] = None

selected_objects: typing.List['bpy.types.Object'] = None

selected_pose_bones: typing.List['bpy.types.PoseBone'] = None

selected_pose_bones_from_active_object: typing.List[
    'bpy.types.PoseBone'] = None

selected_sequences: typing.List['bpy.types.Sequence'] = None

selected_visible_actions: typing.List['bpy.types.Action'] = None

selected_visible_fcurves: typing.List['bpy.types.FCurve'] = None

sequences: typing.List['bpy.types.Sequence'] = None

soft_body: 'bpy.types.SoftBodyModifier' = None

space_data: 'bpy.types.Space' = None

speaker: 'bpy.types.Speaker' = None

texture: 'bpy.types.Texture' = None

texture_slot: 'bpy.types.TextureSlot' = None

texture_user: 'bpy.types.ID' = None

texture_user_property: 'bpy.types.Property' = None

tool_settings: 'bpy.types.ToolSettings' = None

ui_list: 'bpy.types.UIList' = None

vertex_paint_object: 'bpy.types.Object' = None

view_layer: 'bpy.types.ViewLayer' = None

visible_bones: typing.List['bpy.types.EditBone'] = None

visible_fcurves: typing.List['bpy.types.FCurve'] = None

visible_gpencil_layers: typing.List['bpy.types.GPencilLayer'] = None

visible_objects: typing.List['bpy.types.Object'] = None

visible_pose_bones: typing.List['bpy.types.PoseBone'] = None

volume: 'bpy.types.Volume' = None

weight_paint_object: 'bpy.types.Object' = None

window: 'bpy.types.Window' = None

window_manager: 'bpy.types.WindowManager' = None

workspace: 'bpy.types.WorkSpace' = None

world: 'bpy.types.World' = None



import sys
import typing


def fbx(filepath: str = "",
        check_existing: bool = True,
        filter_glob: str = "*.fbx",
        use_selection: bool = False,
        use_visible: bool = False,
        use_active_collection: bool = False,
        global_scale: float = 1.0,
        apply_unit_scale: bool = True,
        apply_scale_options: typing.Union[int, str] = 'FBX_SCALE_NONE',
        use_space_transform: bool = True,
        bake_space_transform: bool = False,
        object_types: typing.Union[typing.Set[int], typing.Set[str]] = {
            'ARMATURE', 'CAMERA', 'EMPTY', 'LIGHT', 'MESH', 'OTHER'
        },
        use_mesh_modifiers: bool = True,
        use_mesh_modifiers_render: bool = True,
        mesh_smooth_type: typing.Union[int, str] = 'OFF',
        use_subsurf: bool = False,
        use_mesh_edges: bool = False,
        use_tspace: bool = False,
        use_triangles: bool = False,
        use_custom_props: bool = False,
        add_leaf_bones: bool = True,
        primary_bone_axis: typing.Union[int, str] = 'Y',
        secondary_bone_axis: typing.Union[int, str] = 'X',
        use_armature_deform_only: bool = False,
        armature_nodetype: typing.Union[int, str] = 'NULL',
        bake_anim: bool = True,
        bake_anim_use_all_bones: bool = True,
        bake_anim_use_nla_strips: bool = True,
        bake_anim_use_all_actions: bool = True,
        bake_anim_force_startend_keying: bool = True,
        bake_anim_step: float = 1.0,
        bake_anim_simplify_factor: float = 1.0,
        path_mode: typing.Union[int, str] = 'AUTO',
        embed_textures: bool = False,
        batch_mode: typing.Union[int, str] = 'OFF',
        use_batch_own_dir: bool = True,
        use_metadata: bool = True,
        axis_forward: typing.Union[int, str] = '-Z',
        axis_up: typing.Union[int, str] = 'Y'):
    ''' Write a FBX file

    :param filepath: File Path, Filepath used for exporting the file
    :type filepath: str
    :param check_existing: Check Existing, Check and warn on overwriting existing files
    :type check_existing: bool
    :param filter_glob: filter_glob
    :type filter_glob: str
    :param use_selection: Selected Objects, Export selected and visible objects only
    :type use_selection: bool
    :param use_visible: Visible Objects, Export visible objects only
    :type use_visible: bool
    :param use_active_collection: Active Collection, Export only objects from the active collection (and its children)
    :type use_active_collection: bool
    :param global_scale: Scale, Scale all data (Some importers do not support scaled armatures!)
    :type global_scale: float
    :param apply_unit_scale: Apply Unit, Take into account current Blender units settings (if unset, raw Blender Units values are used as-is)
    :type apply_unit_scale: bool
    :param apply_scale_options: Apply Scalings, How to apply custom and units scalings in generated FBX file (Blender uses FBX scale to detect units on import, but many other applications do not handle the same way) * FBX_SCALE_NONE All Local -- Apply custom scaling and units scaling to each object transformation, FBX scale remains at 1.0. * FBX_SCALE_UNITS FBX Units Scale -- Apply custom scaling to each object transformation, and units scaling to FBX scale. * FBX_SCALE_CUSTOM FBX Custom Scale -- Apply custom scaling to FBX scale, and units scaling to each object transformation. * FBX_SCALE_ALL FBX All -- Apply custom scaling and units scaling to FBX scale.
    :type apply_scale_options: typing.Union[int, str]
    :param use_space_transform: Use Space Transform, Apply global space transform to the object rotations. When disabled only the axis space is written to the file and all object transforms are left as-is
    :type use_space_transform: bool
    :param bake_space_transform: Apply Transform, Bake space transform into object data, avoids getting unwanted rotations to objects when target space is not aligned with Blender's space (WARNING! experimental option, use at own risks, known broken with armatures/animations)
    :type bake_space_transform: bool
    :param object_types: Object Types, Which kind of object to export * EMPTY Empty. * CAMERA Camera. * LIGHT Lamp. * ARMATURE Armature -- WARNING: not supported in dupli/group instances. * MESH Mesh. * OTHER Other -- Other geometry types, like curve, metaball, etc. (converted to meshes).
    :type object_types: typing.Union[typing.Set[int], typing.Set[str]]
    :param use_mesh_modifiers: Apply Modifiers, Apply modifiers to mesh objects (except Armature ones) - WARNING: prevents exporting shape keys
    :type use_mesh_modifiers: bool
    :param use_mesh_modifiers_render: Use Modifiers Render Setting, Use render settings when applying modifiers to mesh objects (DISABLED in Blender 2.8)
    :type use_mesh_modifiers_render: bool
    :param mesh_smooth_type: Smoothing, Export smoothing information (prefer 'Normals Only' option if your target importer understand split normals) * OFF Normals Only -- Export only normals instead of writing edge or face smoothing data. * FACE Face -- Write face smoothing. * EDGE Edge -- Write edge smoothing.
    :type mesh_smooth_type: typing.Union[int, str]
    :param use_subsurf: Export Subdivision Surface, Export the last Catmull-Rom subdivision modifier as FBX subdivision (does not apply the modifier even if 'Apply Modifiers' is enabled)
    :type use_subsurf: bool
    :param use_mesh_edges: Loose Edges, Export loose edges (as two-vertices polygons)
    :type use_mesh_edges: bool
    :param use_tspace: Tangent Space, Add binormal and tangent vectors, together with normal they form the tangent space (will only work correctly with tris/quads only meshes!)
    :type use_tspace: bool
    :param use_triangles: Triangulate Faces, Convert all faces to triangles
    :type use_triangles: bool
    :param use_custom_props: Custom Properties, Export custom properties
    :type use_custom_props: bool
    :param add_leaf_bones: Add Leaf Bones, Append a final bone to the end of each chain to specify last bone length (use this when you intend to edit the armature from exported data)
    :type add_leaf_bones: bool
    :param primary_bone_axis: Primary Bone Axis
    :type primary_bone_axis: typing.Union[int, str]
    :param secondary_bone_axis: Secondary Bone Axis
    :type secondary_bone_axis: typing.Union[int, str]
    :param use_armature_deform_only: Only Deform Bones, Only write deforming bones (and non-deforming ones when they have deforming children)
    :type use_armature_deform_only: bool
    :param armature_nodetype: Armature FBXNode Type, FBX type of node (object) used to represent Blender's armatures (use Null one unless you experience issues with other app, other choices may no import back perfectly in Blender...) * NULL Null -- 'Null' FBX node, similar to Blender's Empty (default). * ROOT Root -- 'Root' FBX node, supposed to be the root of chains of bones.... * LIMBNODE LimbNode -- 'LimbNode' FBX node, a regular joint between two bones....
    :type armature_nodetype: typing.Union[int, str]
    :param bake_anim: Baked Animation, Export baked keyframe animation
    :type bake_anim: bool
    :param bake_anim_use_all_bones: Key All Bones, Force exporting at least one key of animation for all bones (needed with some target applications, like UE4)
    :type bake_anim_use_all_bones: bool
    :param bake_anim_use_nla_strips: NLA Strips, Export each non-muted NLA strip as a separated FBX's AnimStack, if any, instead of global scene animation
    :type bake_anim_use_nla_strips: bool
    :param bake_anim_use_all_actions: All Actions, Export each action as a separated FBX's AnimStack, instead of global scene animation (note that animated objects will get all actions compatible with them, others will get no animation at all)
    :type bake_anim_use_all_actions: bool
    :param bake_anim_force_startend_keying: Force Start/End Keying, Always add a keyframe at start and end of actions for animated channels
    :type bake_anim_force_startend_keying: bool
    :param bake_anim_step: Sampling Rate, How often to evaluate animated values (in frames)
    :type bake_anim_step: float
    :param bake_anim_simplify_factor: Simplify, How much to simplify baked values (0.0 to disable, the higher the more simplified)
    :type bake_anim_simplify_factor: float
    :param path_mode: Path Mode, Method used to reference paths * AUTO Auto -- Use Relative paths with subdirectories only. * ABSOLUTE Absolute -- Always write absolute paths. * RELATIVE Relative -- Always write relative paths (where possible). * MATCH Match -- Match Absolute/Relative setting with input path. * STRIP Strip Path -- Filename only. * COPY Copy -- Copy the file to the destination path (or subdirectory).
    :type path_mode: typing.Union[int, str]
    :param embed_textures: Embed Textures, Embed textures in FBX binary file (only for "Copy" path mode!)
    :type embed_textures: bool
    :param batch_mode: Batch Mode * OFF Off -- Active scene to file. * SCENE Scene -- Each scene as a file. * COLLECTION Collection -- Each collection (data-block ones) as a file, does not include content of children collections. * SCENE_COLLECTION Scene Collections -- Each collection (including master, non-data-block ones) of each scene as a file, including content from children collections. * ACTIVE_SCENE_COLLECTION Active Scene Collections -- Each collection (including master, non-data-block one) of the active scene as a file, including content from children collections.
    :type batch_mode: typing.Union[int, str]
    :param use_batch_own_dir: Batch Own Dir, Create a dir for each exported file
    :type use_batch_own_dir: bool
    :param use_metadata: Use Metadata
    :type use_metadata: bool
    :param axis_forward: Forward
    :type axis_forward: typing.Union[int, str]
    :param axis_up: Up
    :type axis_up: typing.Union[int, str]
    '''

    pass


def gltf(filepath: str = "",
         check_existing: bool = True,
         export_format: typing.Union[int, str] = 'GLB',
         ui_tab: typing.Union[int, str] = 'GENERAL',
         export_copyright: str = "",
         export_image_format: typing.Union[int, str] = 'AUTO',
         export_texture_dir: str = "",
         export_keep_originals: bool = False,
         export_texcoords: bool = True,
         export_normals: bool = True,
         export_draco_mesh_compression_enable: bool = False,
         export_draco_mesh_compression_level: int = 6,
         export_draco_position_quantization: int = 14,
         export_draco_normal_quantization: int = 10,
         export_draco_texcoord_quantization: int = 12,
         export_draco_color_quantization: int = 10,
         export_draco_generic_quantization: int = 12,
         export_tangents: bool = False,
         export_materials: typing.Union[int, str] = 'EXPORT',
         export_colors: bool = True,
         use_mesh_edges: bool = False,
         use_mesh_vertices: bool = False,
         export_cameras: bool = False,
         use_selection: bool = False,
         use_visible: bool = False,
         use_renderable: bool = False,
         use_active_collection: bool = False,
         use_active_scene: bool = False,
         export_extras: bool = False,
         export_yup: bool = True,
         export_apply: bool = False,
         export_animations: bool = True,
         export_frame_range: bool = True,
         export_frame_step: int = 1,
         export_force_sampling: bool = True,
         export_nla_strips: bool = True,
         export_nla_strips_merged_animation_name: str = "Animation",
         export_def_bones: bool = False,
         optimize_animation_size: bool = False,
         export_current_frame: bool = False,
         export_skins: bool = True,
         export_all_influences: bool = False,
         export_morph: bool = True,
         export_morph_normal: bool = True,
         export_morph_tangent: bool = False,
         export_lights: bool = False,
         export_displacement: bool = False,
         will_save_settings: bool = False,
         filter_glob: str = "*.glb;*.gltf"):
    ''' Export scene as glTF 2.0 file

    :param filepath: File Path, Filepath used for exporting the file
    :type filepath: str
    :param check_existing: Check Existing, Check and warn on overwriting existing files
    :type check_existing: bool
    :param export_format: Format, Output format and embedding options. Binary is most efficient, but JSON (embedded or separate) may be easier to edit later * GLB glTF Binary (.glb) -- Exports a single file, with all data packed in binary form. Most efficient and portable, but more difficult to edit later. * GLTF_SEPARATE glTF Separate (.gltf + .bin + textures) -- Exports multiple files, with separate JSON, binary and texture data. Easiest to edit later. * GLTF_EMBEDDED glTF Embedded (.gltf) -- Exports a single file, with all data packed in JSON. Less efficient than binary, but easier to edit later.
    :type export_format: typing.Union[int, str]
    :param ui_tab: ui_tab, Export setting categories * GENERAL General -- General settings. * MESHES Meshes -- Mesh settings. * OBJECTS Objects -- Object settings. * ANIMATION Animation -- Animation settings.
    :type ui_tab: typing.Union[int, str]
    :param export_copyright: Copyright, Legal rights and conditions for the model
    :type export_copyright: str
    :param export_image_format: Images, Output format for images. PNG is lossless and generally preferred, but JPEG might be preferable for web applications due to the smaller file size. Alternatively they can be omitted if they are not needed * AUTO Automatic -- Save PNGs as PNGs and JPEGs as JPEGs. If neither one, use PNG. * JPEG JPEG Format (.jpg) -- Save images as JPEGs. (Images that need alpha are saved as PNGs though.) Be aware of a possible loss in quality. * NONE None -- Don't export images.
    :type export_image_format: typing.Union[int, str]
    :param export_texture_dir: Textures, Folder to place texture files in. Relative to the .gltf file
    :type export_texture_dir: str
    :param export_keep_originals: Keep original, Keep original textures files if possible. WARNING: if you use more than one texture, where pbr standard requires only one, only one texture will be used. This can lead to unexpected results
    :type export_keep_originals: bool
    :param export_texcoords: UVs, Export UVs (texture coordinates) with meshes
    :type export_texcoords: bool
    :param export_normals: Normals, Export vertex normals with meshes
    :type export_normals: bool
    :param export_draco_mesh_compression_enable: Draco mesh compression, Compress mesh using Draco
    :type export_draco_mesh_compression_enable: bool
    :param export_draco_mesh_compression_level: Compression level, Compression level (0 = most speed, 6 = most compression, higher values currently not supported)
    :type export_draco_mesh_compression_level: int
    :param export_draco_position_quantization: Position quantization bits, Quantization bits for position values (0 = no quantization)
    :type export_draco_position_quantization: int
    :param export_draco_normal_quantization: Normal quantization bits, Quantization bits for normal values (0 = no quantization)
    :type export_draco_normal_quantization: int
    :param export_draco_texcoord_quantization: Texcoord quantization bits, Quantization bits for texture coordinate values (0 = no quantization)
    :type export_draco_texcoord_quantization: int
    :param export_draco_color_quantization: Color quantization bits, Quantization bits for color values (0 = no quantization)
    :type export_draco_color_quantization: int
    :param export_draco_generic_quantization: Generic quantization bits, Quantization bits for generic coordinate values like weights or joints (0 = no quantization)
    :type export_draco_generic_quantization: int
    :param export_tangents: Tangents, Export vertex tangents with meshes
    :type export_tangents: bool
    :param export_materials: Materials, Export materials * EXPORT Export -- Export all materials used by included objects. * PLACEHOLDER Placeholder -- Do not export materials, but write multiple primitive groups per mesh, keeping material slot information. * NONE No export -- Do not export materials, and combine mesh primitive groups, losing material slot information.
    :type export_materials: typing.Union[int, str]
    :param export_colors: Vertex Colors, Export vertex colors with meshes
    :type export_colors: bool
    :param use_mesh_edges: Loose Edges, Export loose edges as lines, using the material from the first material slot
    :type use_mesh_edges: bool
    :param use_mesh_vertices: Loose Points, Export loose points as glTF points, using the material from the first material slot
    :type use_mesh_vertices: bool
    :param export_cameras: Cameras, Export cameras
    :type export_cameras: bool
    :param use_selection: Selected Objects, Export selected objects only
    :type use_selection: bool
    :param use_visible: Visible Objects, Export visible objects only
    :type use_visible: bool
    :param use_renderable: Renderable Objects, Export renderable objects only
    :type use_renderable: bool
    :param use_active_collection: Active Collection, Export objects in the active collection only
    :type use_active_collection: bool
    :param use_active_scene: Active Scene, Export active scene only
    :type use_active_scene: bool
    :param export_extras: Custom Properties, Export custom properties as glTF extras
    :type export_extras: bool
    :param export_yup: +Y Up, Export using glTF convention, +Y up
    :type export_yup: bool
    :param export_apply: Apply Modifiers, Apply modifiers (excluding Armatures) to mesh objects -WARNING: prevents exporting shape keys
    :type export_apply: bool
    :param export_animations: Animations, Exports active actions and NLA tracks as glTF animations
    :type export_animations: bool
    :param export_frame_range: Limit to Playback Range, Clips animations to selected playback range
    :type export_frame_range: bool
    :param export_frame_step: Sampling Rate, How often to evaluate animated values (in frames)
    :type export_frame_step: int
    :param export_force_sampling: Always Sample Animations, Apply sampling to all animations
    :type export_force_sampling: bool
    :param export_nla_strips: Group by NLA Track, When on, multiple actions become part of the same glTF animation if they're pushed onto NLA tracks with the same name. When off, all the currently assigned actions become one glTF animation
    :type export_nla_strips: bool
    :param export_nla_strips_merged_animation_name: Merged Animation Name, Name of single glTF animation to be exported
    :type export_nla_strips_merged_animation_name: str
    :param export_def_bones: Export Deformation Bones Only, Export Deformation bones only
    :type export_def_bones: bool
    :param optimize_animation_size: Optimize Animation Size, Reduce exported file-size by removing duplicate keyframes(can cause problems with stepped animation)
    :type optimize_animation_size: bool
    :param export_current_frame: Use Current Frame, Export the scene in the current animation frame
    :type export_current_frame: bool
    :param export_skins: Skinning, Export skinning (armature) data
    :type export_skins: bool
    :param export_all_influences: Include All Bone Influences, Allow >4 joint vertex influences. Models may appear incorrectly in many viewers
    :type export_all_influences: bool
    :param export_morph: Shape Keys, Export shape keys (morph targets)
    :type export_morph: bool
    :param export_morph_normal: Shape Key Normals, Export vertex normals with shape keys (morph targets)
    :type export_morph_normal: bool
    :param export_morph_tangent: Shape Key Tangents, Export vertex tangents with shape keys (morph targets)
    :type export_morph_tangent: bool
    :param export_lights: Punctual Lights, Export directional, point, and spot lights. Uses "KHR_lights_punctual" glTF extension
    :type export_lights: bool
    :param export_displacement: Displacement Textures (EXPERIMENTAL), EXPERIMENTAL: Export displacement textures. Uses incomplete "KHR_materials_displacement" glTF extension
    :type export_displacement: bool
    :param will_save_settings: Remember Export Settings, Store glTF export settings in the Blender project
    :type will_save_settings: bool
    :param filter_glob: filter_glob
    :type filter_glob: str
    '''

    pass


def obj(filepath: str = "",
        check_existing: bool = True,
        filter_glob: str = "*.obj;*.mtl",
        use_selection: bool = False,
        use_animation: bool = False,
        use_mesh_modifiers: bool = True,
        use_edges: bool = True,
        use_smooth_groups: bool = False,
        use_smooth_groups_bitflags: bool = False,
        use_normals: bool = True,
        use_uvs: bool = True,
        use_materials: bool = True,
        use_triangles: bool = False,
        use_nurbs: bool = False,
        use_vertex_groups: bool = False,
        use_blen_objects: bool = True,
        group_by_object: bool = False,
        group_by_material: bool = False,
        keep_vertex_order: bool = False,
        global_scale: float = 1.0,
        path_mode: typing.Union[int, str] = 'AUTO',
        axis_forward: typing.Union[int, str] = '-Z',
        axis_up: typing.Union[int, str] = 'Y'):
    ''' Save a Wavefront OBJ File

    :param filepath: File Path, Filepath used for exporting the file
    :type filepath: str
    :param check_existing: Check Existing, Check and warn on overwriting existing files
    :type check_existing: bool
    :param filter_glob: filter_glob
    :type filter_glob: str
    :param use_selection: Selection Only, Export selected objects only
    :type use_selection: bool
    :param use_animation: Animation, Write out an OBJ for each frame
    :type use_animation: bool
    :param use_mesh_modifiers: Apply Modifiers, Apply modifiers
    :type use_mesh_modifiers: bool
    :param use_edges: Include Edges
    :type use_edges: bool
    :param use_smooth_groups: Smooth Groups, Write sharp edges as smooth groups
    :type use_smooth_groups: bool
    :param use_smooth_groups_bitflags: Bitflag Smooth Groups, Same as 'Smooth Groups', but generate smooth groups IDs as bitflags (produces at most 32 different smooth groups, usually much less)
    :type use_smooth_groups_bitflags: bool
    :param use_normals: Write Normals, Export one normal per vertex and per face, to represent flat faces and sharp edges
    :type use_normals: bool
    :param use_uvs: Include UVs, Write out the active UV coordinates
    :type use_uvs: bool
    :param use_materials: Write Materials, Write out the MTL file
    :type use_materials: bool
    :param use_triangles: Triangulate Faces, Convert all faces to triangles
    :type use_triangles: bool
    :param use_nurbs: Write Nurbs, Write nurbs curves as OBJ nurbs rather than converting to geometry
    :type use_nurbs: bool
    :param use_vertex_groups: Polygroups
    :type use_vertex_groups: bool
    :param use_blen_objects: OBJ Objects, Export Blender objects as OBJ objects
    :type use_blen_objects: bool
    :param group_by_object: OBJ Groups, Export Blender objects as OBJ groups
    :type group_by_object: bool
    :param group_by_material: Material Groups, Generate an OBJ group for each part of a geometry using a different material
    :type group_by_material: bool
    :param keep_vertex_order: Keep Vertex Order
    :type keep_vertex_order: bool
    :param global_scale: Scale
    :type global_scale: float
    :param path_mode: Path Mode, Method used to reference paths * AUTO Auto -- Use Relative paths with subdirectories only. * ABSOLUTE Absolute -- Always write absolute paths. * RELATIVE Relative -- Always write relative paths (where possible). * MATCH Match -- Match Absolute/Relative setting with input path. * STRIP Strip Path -- Filename only. * COPY Copy -- Copy the file to the destination path (or subdirectory).
    :type path_mode: typing.Union[int, str]
    :param axis_forward: Forward
    :type axis_forward: typing.Union[int, str]
    :param axis_up: Up
    :type axis_up: typing.Union[int, str]
    '''

    pass


def x3d(filepath: str = "",
        check_existing: bool = True,
        filter_glob: str = "*.x3d",
        use_selection: bool = False,
        use_mesh_modifiers: bool = True,
        use_triangulate: bool = False,
        use_normals: bool = False,
        use_compress: bool = False,
        use_hierarchy: bool = True,
        name_decorations: bool = True,
        use_h3d: bool = False,
        global_scale: float = 1.0,
        path_mode: typing.Union[int, str] = 'AUTO',
        axis_forward: typing.Union[int, str] = 'Z',
        axis_up: typing.Union[int, str] = 'Y'):
    ''' Export selection to Extensible 3D file (.x3d)

    :param filepath: File Path, Filepath used for exporting the file
    :type filepath: str
    :param check_existing: Check Existing, Check and warn on overwriting existing files
    :type check_existing: bool
    :param filter_glob: filter_glob
    :type filter_glob: str
    :param use_selection: Selection Only, Export selected objects only
    :type use_selection: bool
    :param use_mesh_modifiers: Apply Modifiers, Use transformed mesh data from each object
    :type use_mesh_modifiers: bool
    :param use_triangulate: Triangulate, Write quads into 'IndexedTriangleSet'
    :type use_triangulate: bool
    :param use_normals: Normals, Write normals with geometry
    :type use_normals: bool
    :param use_compress: Compress, Compress the exported file
    :type use_compress: bool
    :param use_hierarchy: Hierarchy, Export parent child relationships
    :type use_hierarchy: bool
    :param name_decorations: Name decorations, Add prefixes to the names of exported nodes to indicate their type
    :type name_decorations: bool
    :param use_h3d: H3D Extensions, Export shaders for H3D
    :type use_h3d: bool
    :param global_scale: Scale
    :type global_scale: float
    :param path_mode: Path Mode, Method used to reference paths * AUTO Auto -- Use Relative paths with subdirectories only. * ABSOLUTE Absolute -- Always write absolute paths. * RELATIVE Relative -- Always write relative paths (where possible). * MATCH Match -- Match Absolute/Relative setting with input path. * STRIP Strip Path -- Filename only. * COPY Copy -- Copy the file to the destination path (or subdirectory).
    :type path_mode: typing.Union[int, str]
    :param axis_forward: Forward
    :type axis_forward: typing.Union[int, str]
    :param axis_up: Up
    :type axis_up: typing.Union[int, str]
    '''

    pass



import sys
import typing


def convert_from_particle_system():
    ''' Add a new curves object based on the current state of the particle system

    '''

    pass


def convert_to_particle_system():
    ''' Add a new or update an existing hair particle system on the surface object

    '''

    pass


def disable_selection():
    ''' Disable the drawing of influence of selection in sculpt mode

    '''

    pass


def sculptmode_toggle():
    ''' Enter/Exit sculpt mode for curves

    '''

    pass


def set_selection_domain(domain: typing.Union[int, str] = 'POINT'):
    ''' Change the mode used for selection masking in curves sculpt mode

    :param domain: Domain
    :type domain: typing.Union[int, str]
    '''

    pass


def snap_curves_to_surface(attach_mode: typing.Union[int, str] = 'NEAREST'):
    ''' Move curves so that the first point is exactly on the surface mesh

    :param attach_mode: Attach Mode, How to find the point on the surface to attach to * NEAREST Nearest -- Find the closest point on the surface for the root point of every curve and move the root there. * DEFORM Deform -- Re-attach curves to a deformed surface using the existing attachment information. This only works when the topology of the surface mesh has not changed.
    :type attach_mode: typing.Union[int, str]
    '''

    pass



import sys
import typing


def rule_add(type: typing.Union[int, str] = 'GOAL'):
    ''' Add a boid rule to the current boid state

    :param type: Type
    :type type: typing.Union[int, str]
    '''

    pass


def rule_del():
    ''' Delete current boid rule

    '''

    pass


def rule_move_down():
    ''' Move boid rule down in the list

    '''

    pass


def rule_move_up():
    ''' Move boid rule up in the list

    '''

    pass


def state_add():
    ''' Add a boid state to the particle system

    '''

    pass


def state_del():
    ''' Delete current boid state

    '''

    pass


def state_move_down():
    ''' Move boid state down in the list

    '''

    pass


def state_move_up():
    ''' Move boid state up in the list

    '''

    pass



import sys
import typing
has_joystick: bool = None
''' True if the BGE has been built with joystick support.
'''

has_physics: bool = None
''' True if the BGE has been built with physics support.
'''

has_texture_ffmpeg: bool = None
''' True if the BGE has been built with FFmpeg support, enabling use of ~bge.texture.ImageFFmpeg and ~bge.texture.VideoFFmpeg .
'''

upbge_version: tuple = None
''' The UPBGE version as a tuple of 3 ints, eg. (0, 0, 3).
'''

upbge_version_string: str = None
''' The UPBGE version formatted as a string, eg. "0.0 (sub 3)".
'''

version: tuple = None
''' The Blender/BGE version as a tuple of 3 ints, eg. (2, 75, 1).
'''

version_char: str = None
''' The Blender/BGE version character (for minor releases).
'''

version_string: str = None
''' The Blender/BGE version formatted as a string, eg. "2.75 (sub 1)".
'''



import sys
import typing


def autoDebugList(enable: bool):
    ''' Enable or disable auto adding debug properties to the debug list.

    :type enable: bool
    '''

    pass


def clearDebugList():
    ''' Clears the debug property list.

    '''

    pass


def disableMotionBlur():
    ''' Disable the motion blur effect.

    '''

    pass


def drawLine(fromVec: list, toVec: list, color: list):
    ''' Draw a line in the 3D scene.

    :param fromVec: the origin of the line
    :type fromVec: list
    :param toVec: the end of the line
    :type toVec: list
    :param color: the color of the line
    :type color: list
    '''

    pass


def enableMotionBlur(factor: float):
    ''' Enable the motion blur effect.

    :param factor: the ammount of motion blur to display.
    :type factor: float
    '''

    pass


def enableVisibility(visible):
    ''' 

    '''

    pass


def getAnisotropicFiltering():
    ''' Get the anisotropic filtering level used for textures.

    '''

    pass


def getDisplayDimensions():
    ''' Get the display dimensions, in pixels, of the display (e.g., the monitor). Can return the size of the entire view, so the combination of all monitors; for example, (3840, 1080) for two side-by-side 1080p monitors.

    '''

    pass


def getEyeSeparation():
    ''' Gets the current eye separation for stereo mode.

    '''

    pass


def getFocalLength():
    ''' Gets the current focal length for stereo mode.

    '''

    pass


def getFullScreen():
    ''' Returns whether or not the window is fullscreen.

    '''

    pass


def getGLSLMaterialSetting(setting):
    ''' 

    '''

    pass


def getMaterialMode(mode):
    ''' 

    '''

    pass


def getMipmapping():
    ''' Get the current mipmapping setting.

    :return: ~bge.render.RAS_MIPMAP_LINEAR
    '''

    pass


def getStereoEye() -> typing.Union['RIGHT_EYE', 'LEFT_EYE']:
    ''' Gets the current stereoscopy eye being rendered. This function is mainly used in a :attr: bge.types.KX_Scene.pre_draw callback function to customize the camera projection matrices for each stereoscopic eye.

    :rtype: typing.Union['RIGHT_EYE', 'LEFT_EYE']
    :return: ~bge.render.RIGHT_EYE .
    '''

    pass


def getVsync():
    ''' Get the current vsync value

    :return: ~bge.render.VSYNC_ADAPTIVE
    '''

    pass


def getWindowHeight():
    ''' Gets the height of the window (in pixels)

    '''

    pass


def getWindowWidth():
    ''' Gets the width of the window (in pixels)

    '''

    pass


def makeScreenshot(filename: str):
    ''' Writes an image file with the displayed image at the frame end. The image is written to *'filename'*. The path may be absolute (eg. /home/foo/image ) or relative when started with // (eg. //image ). Note that absolute paths are not portable between platforms. If the filename contains a # , it will be replaced by an incremental index so that screenshots can be taken multiple times without overwriting the previous ones (eg. image-# ). Settings for the image are taken from the render settings (file format and respective settings, gamma and colospace conversion, etc). The image resolution matches the framebuffer, meaning, the window size and aspect ratio. When running from the standalone player, instead of the embedded player, only PNG files are supported. Additional color conversions are also not supported.

    :param filename: path and name of the file to write
    :type filename: str
    '''

    pass


def setAnisotropicFiltering(level):
    ''' Set the anisotropic filtering level for textures.

    :param level: The new anisotropic filtering level to use
    '''

    pass


def setBackgroundColor(rgba):
    ''' 

    '''

    pass


def setEyeSeparation(eyesep: float):
    ''' Sets the eye separation for stereo mode. Usually Focal Length/30 provides a confortable value.

    :param eyesep: The distance between the left and right eye.
    :type eyesep: float
    '''

    pass


def setFocalLength(focallength: float):
    ''' Sets the focal length for stereo mode. It uses the current camera focal length as initial value.

    :param focallength: The focal length.
    :type focallength: float
    '''

    pass


def setFullScreen(enable: bool):
    ''' Set whether or not the window should be fullscreen.

    :param enable: True to set full screen, False to set windowed.
    :type enable: bool
    '''

    pass


def setGLSLMaterialSetting(setting, enable):
    ''' 

    '''

    pass


def setMaterialMode(mode):
    ''' 

    '''

    pass


def setMipmapping(value):
    ''' Change how to use mipmapping.

    :param value: ~bge.render.RAS_MIPMAP_LINEAR
    '''

    pass


def setMousePosition(x, y):
    ''' Sets the mouse cursor position.

    :param x: X-coordinate in screen pixel coordinates.
    :param y: Y-coordinate in screen pixel coordinates.
    '''

    pass


def setVsync(value):
    ''' Set the vsync value

    :param value: ~bge.render.VSYNC_ADAPTIVE
    '''

    pass


def setWindowSize(width, height):
    ''' Set the width and height of the window (in pixels). This also works for fullscreen applications.

    :param width: width in pixels
    :param height: height in pixels
    '''

    pass


def showFramerate(enable: bool):
    ''' Show or hide the framerate.

    :type enable: bool
    '''

    pass


def showMouse(visible: bool):
    ''' Enables or disables the operating system mouse cursor.

    :type visible: bool
    '''

    pass


def showProfile(enable: bool):
    ''' Show or hide the profile.

    :type enable: bool
    '''

    pass


def showProperties(enable: bool):
    ''' Show or hide the debug properties.

    :type enable: bool
    '''

    pass


KX_BLENDER_GLSL_MATERIAL = None

KX_BLENDER_MULTITEX_MATERIAL = None

KX_TEXFACE_MATERIAL = None

LEFT_EYE = None
''' Left eye being used during stereoscopic rendering.
'''

RAS_MIPMAP_LINEAR = None
''' Applies mipmap filtering with nearest linear interpolation.
'''

RAS_MIPMAP_NEAREST = None
''' Applies mipmap filtering with nearest neighbour interpolation.
'''

RAS_MIPMAP_NONE = None
''' Disables Mipmap filtering.
'''

RIGHT_EYE = None
''' Right eye being used during stereoscopic rendering.
'''

VSYNC_ADAPTIVE = None
''' Enables adaptive vsync if supported. Adaptive vsync enables vsync if the framerate is above the monitors refresh rate. Otherwise, vsync is diabled if the framerate is too low.
'''

VSYNC_OFF = None
''' Disables vsync
'''

VSYNC_ON = None
''' Enables vsync
'''



import sys
import typing


class I18nSettings:
    BRANCHES_DIR = None
    ''' '''

    FILE_NAME_POT = None
    ''' '''

    GIT_I18N_PO_DIR = None
    ''' '''

    GIT_I18N_ROOT = None
    ''' '''

    MO_PATH_ROOT = None
    ''' '''

    MO_PATH_TEMPLATE = None
    ''' '''

    POTFILES_SOURCE_DIR = None
    ''' '''

    PY_SYS_PATHS = None
    ''' '''

    TRUNK_DIR = None
    ''' '''

    TRUNK_MO_DIR = None
    ''' '''

    TRUNK_PO_DIR = None
    ''' '''

    def from_dict(self, mapping):
        ''' 

        '''
        pass

    def from_json(self, string):
        ''' 

        '''
        pass

    def load(self, fname, reset):
        ''' 

        '''
        pass

    def save(self, fname):
        ''' 

        '''
        pass

    def to_dict(self):
        ''' 

        '''
        pass

    def to_json(self):
        ''' 

        '''
        pass
